#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.
#shellcheck disable=SC1090,SC1091,SC2034

source "${MODULES_DIR}"/utils.sh

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/cluster_utils.sh

TARGET_BASE_PATH="/opt/UiPathAutomationSuite"
TARGET_TEMP_PATH="${TARGET_BASE_PATH}/tmp"
HADR_CERTS_FOLDER_PATH="${TARGET_TEMP_PATH}/hadr_certs"
ALTERNATE_CLUSTER_KUBECONFIG="${TARGET_TEMP_PATH}/hadr.kubeconfig"

function capture_hadr_override_mandatory_parameter_value() {
  local filepath="${1}"
  local key="${2}"
  local value

  value=$(jq -r ".spec.source.helm.parameters[] | select(.name==\"${key}\").value" < "${filepath}")
  if [[ -n "${value}" && ${value} != null ]]; then
    echo "${value}"
  else
    error "${key} not found in input file ${filepath}"
  fi
}

function capture_hadr_override_optional_parameter_value() {
  local filepath="${1}"
  local key="${2}"
  local value

  value=$(jq -r ".spec.source.helm.parameters[] | select(.name==\"${key}\").value" < "${filepath}")
  if [[ -n "${value}" && ${value} != null ]]; then
    echo "${value}"
  else
    echo ""
  fi
}

function hadr_params_override() {
  local primary_kubeconfig="${SCRIPT_DIR}/alternate-cluster-access.yaml"
  local argocd_app_json_file="${SCRIPT_DIR}/primary_uipath_app.json"

  kubectl get secret -n uipath-infra alternate-cluster-access -o jsonpath='{.data.alternate-cluster-kubeconfig}' | \
    base64 -d > "${primary_kubeconfig}" || error_with_code "15012"

  kubectl --kubeconfig="${primary_kubeconfig}" get applications -n argocd uipath -o json > "${argocd_app_json_file}" || \
    error_with_code "15016"

  ORCHESTRATOR_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.sqlConnectionString")"
  ORCHESTRATOR_TESTAUTOMATION_ENABLED="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.testautomation.enabled")"
  ORCHESTRATOR_TA_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.testautomation.sqlConnectionString")"
  ORCHESTRATOR_UPDATESERVER_ENABLED="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.updateserver.enabled")"
  ORCHESTRATOR_UPD_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.updateserver.sqlConnectionString")"
  ORCHESTRATOR_ROBOT_LOGS_ELASTIC_URI="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.orchestratorRobotLogsElasticUri")"
  ORCHESTRATOR_ROBOT_LOGS_ELASTIC_AUTH_USERNAME="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.orchestratorRobotLogsElasticAuthUsername")"
  ORCHESTRATOR_ROBOT_LOGS_ELASTIC_AUTH_PASSWORD="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.orchestratorRobotLogsElasticAuthPassword")"
  IDENTITY_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.identity.sqlConnectionString")"
  IDENTITY_HOST_ADMIN_USERNAME="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.identity.hostAdminUsername")"
  IDENTITY_HOST_ADMIN_PASSWORD="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.identity.hostAdminPassword")"
  IDENTITY_KRB5_KEYTAB="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.identity.krb5KeytabSecret")"
  OMS_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.oms.sqlConnectionString")"
  LOCATION_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.la.sqlConnectionString")"
  LRM_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.lrm.sqlConnectionString")"
  AUDIT_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.audit.sqlConnectionString")"
  RESOURCECATALOG_SQL_CONNECTION="$(capture_hadr_override_mandatory_parameter_value "${argocd_app_json_file}" "global.userInputs.resourcecatalog.sqlConnectionString")"

  ORCHESTRATOR_KERBEROS_AD_USER="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.kerberosAuthConfig.adUserName")"
  ORCHESTRATOR_KERBEROS_KEYTAB="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.orchestrator.kerberosAuthConfig.userKeytab")"
  PLATFORM_KERBEROS_AD_USER="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.platform.kerberosAuthConfig.adUserName")"
  PLATFORM_KERBEROS_KEYTAB="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.platform.kerberosAuthConfig.userKeytab")"
  IDENTITY_SAML_CURRENT_SERVICE_CERT_CONTENT="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.certificate.identity.saml.currentServiceCert")"
  IDENTITY_SAML_CURRENT_SERVICE_CERT_PASS="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.certificate.identity.saml.currentServiceCertPassword")"
  IDENTITY_SAML_FUTURE_SERVICE_CERT_CONTENT="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.certificate.identity.saml.futureServiceCert")"
  IDENTITY_SAML_FUTURE_SERVICE_CERT_PASS="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.certificate.identity.saml.futureServiceCertPassword")"
  IDENTITY_LDAPS_CUSTOM_ROOTCA_CONTENT="$(capture_hadr_override_optional_parameter_value "${argocd_app_json_file}" "global.userInputs.certificate.identity.ldaps.customRootCA")"

  rm -rf "${primary_kubeconfig}"
}

function sync_secrets_configmaps_from_primary() {
  local primary_kubeconfig="${SCRIPT_DIR}/alternate-cluster-access.yaml"
  local non_argo_managed_secrets=("identity-client-audit" "identity-generated-secrets" "identity-token-signing-certificate" "identity-client-robot" "orchestrator-generated-secrets" "identity-client-orchestrator" "identity-client-la" "identity-client-lrm" "identity-client-oms")
  local non_argo_managed_configmaps=("orchestrator-customconfig" "identity-service-customconfig" "location-service-configmap")

  kubectl get secret -n uipath-infra alternate-cluster-access -o jsonpath='{.data.alternate-cluster-kubeconfig}' | \
    base64 -d > "${primary_kubeconfig}" || error_with_code "15012"

  for sec in "${non_argo_managed_secrets[@]}";
  do
    kubectl --kubeconfig="${primary_kubeconfig}" get secrets -n uipath "${sec}" -ojson | jq 'del(.metadata.uid, .metadata.resourceVersion, .metadata.creationTimestamp)' | kubectl apply -f -;
  done

  for cnfMap in "${non_argo_managed_configmaps[@]}";
  do
    kubectl --kubeconfig="${primary_kubeconfig}" get configmap -n uipath "${cnfMap}" -ojson | jq 'del(.metadata.uid, .metadata.resourceVersion, .metadata.creationTimestamp)' | kubectl apply -f -;
  done
}

function create_hadr_rolebinding() {
  local hadr_user="${2}"
  local ROLEBINDING_FILE="${MODULES_DIR}/active-active-rolebinding.yaml"

  kubectl delete rolebinding hadr-kube-system-binding -n kube-system --ignore-not-found
  kubectl delete rolebinding hadr-uipath-binding -n uipath --ignore-not-found
  kubectl delete rolebinding hadr-uipath-infra-binding -n uipath --ignore-not-found
  kubectl delete rolebinding hadr-redis-system-binding -n redis-system --ignore-not-found
  kubectl delete rolebinding hadr-rook-ceph-binding -n rook-ceph --ignore-not-found
  kubectl delete rolebinding hadr-istio-system-binding -n istio-system --ignore-not-found
  kubectl delete clusterrole hadr-role --ignore-not-found

  kubectl apply -f "${ROLEBINDING_FILE}" || error_with_code "15006"

  kubectl create rolebinding hadr-redis-system-binding --namespace=redis-system \
    --clusterrole=hadr-role --user="${hadr_user}"
  kubectl create rolebinding hadr-rook-ceph-binding --namespace=rook-ceph \
    --clusterrole=hadr-role --user="${hadr_user}"
  kubectl create rolebinding hadr-uipath-binding --namespace=uipath \
    --clusterrole=hadr-role --user="${hadr_user}"
  kubectl create rolebinding hadr-uipath-infra-binding --namespace=uipath-infra \
    --clusterrole=hadr-role --user="${hadr_user}"
  kubectl create rolebinding hadr-kube-system-binding --namespace=kube-system \
    --clusterrole=hadr-role --user="${hadr_user}"
  kubectl create rolebinding hadr-istio-system-binding --namespace=istio-system \
    --clusterrole=hadr-role --user="${hadr_user}"
}

function update_zone_keys_on_primary() {
  local secret_key access_key

  info "Update rook-ceph zone keys on the primary cluster"

  rook_tools_pod=$(kubectl -n rook-ceph get pods -l app=rook-ceph-tools \
                      -o jsonpath='{.items[0].metadata.name}' \
                      --ignore-not-found)

  if [[ -n "${rook_tools_pod}" ]]; then
    info "Capturing secret key for rook-ceph-system-user"
    secret_key=$(kubectl -n rook-ceph exec deploy/rook-ceph-tools -- \
      bash -c "radosgw-admin user info --uid rook-ceph-system-user" | \
        jq -r '.keys[0].secret_key' -r) || error_with_code 15010
    info "Capturing access key for rook-ceph-system-user"
    access_key=$(kubectl -n rook-ceph exec deploy/rook-ceph-tools -- \
      bash -c "radosgw-admin user info --uid rook-ceph-system-user" | \
        jq -r '.keys[0].access_key' -r) || error_with_code 15010
    info "Updating secret and access keys for zone rook-ceph"
    kubectl -n rook-ceph exec deploy/rook-ceph-tools -- \
      bash -c "radosgw-admin zone modify --rgw-zone=rook-ceph \
        --access-key=$access_key --secret=$secret_key;  radosgw-admin period update --commit \
        --rgw-realm=rook-ceph --rgw-zonegroup=rook-ceph --rgw-zone=rook-ceph"  || error_with_code 15010
  else
    error_with_code 14022
  fi
}

function create_user_csr() {
  local csr_name="${1}"
  local cert_path="${2}"
  local cert_key="${3}"
  local user="${4}"
  local k8s_csr="${cert_path}/k8s_csr.yaml"
  local csr_file="${cert_path}/csr.yaml"
  local tls_cert_subj="/C=US/ST=NY/O=UiPath, Inc./CN=${user}"
  local csr_request_b64

  openssl req -new -key "${cert_key}" -out "${csr_file}" -subj "${tls_cert_subj}"
  csr_request_b64=$(base64 < "${csr_file}" | tr -d '\n')

  # Create k8s CSR
  cat "${MODULES_DIR}/certificate_signing_request.yaml" > "${k8s_csr}"
  sed -i "s;CSR_NAME_PLACEHOLDER;${csr_name};" "${k8s_csr}"
  sed -i "s;CSR_BASE64_PLACEHOLDER;${csr_request_b64};" "${k8s_csr}"

  kubectl delete csr "${csr_name}" -n default --ignore-not-found
  kubectl create -f "${k8s_csr}" || error_with_code "15000"
  return 0
}

function approve_csr() {
  local csr_name="${1}"
  local try=0
  local maxtry=10
  local issued="false"
  local signed_csr

  kubectl certificate approve "${csr_name}" || error_with_code "15001"

  while [[ ${issued} == "false" ]]  && (( try != maxtry )); do
    try=$((try+1))
    info "Looking for hadr user csr signing...${try}/${maxtry}"
    signed_csr=$(kubectl -n default get csr "${csr_name}" -o jsonpath='{.status.conditions[0].type}' --ignore-not-found)
    info "CSR signing output: ${signed_csr}"
    [[ ${signed_csr} == "Approved" ]] && issued="true"
    [[ $issued != "true" ]] && sleep 30
  done

  [[ $issued != "true" ]] && error_with_code "15001"
  return 0
}

function create_custom_kubeconfig() {
  local csr_name="${1}"
  local cert_path="${2}"
  local cert_key="${3}"
  local cert_user="${4}"
  local hadr_context="hadr-context"
  local hadr_user_crt hadr_ca_crt cluster_name

  hadr_user_crt="${cert_path}/hadr-user-k8s-access.crt"
  hadr_ca_crt="${cert_path}/k8s_ca.crt"

  # Capture user cert from signed csr
  kubectl -n default get csr "${csr_name}" -o jsonpath='{.status.certificate}' | \
    base64 --decode > "${hadr_user_crt}"
  # Capture CA cert from current kubeconfig
  kubectl config view -o jsonpath='{.clusters[0].cluster.certificate-authority-data}' --raw  | \
    base64 --decode > "${hadr_ca_crt}"

  cluster_fqdn="$(jq -r '.alternative_fqdn' < "${CLUSTER_CONFIG_FILE}")"
  if [[ -z "${cluster_fqdn}" ]]; then
    cluster_fqdn="$(jq -r '.fqdn' < "${CLUSTER_CONFIG_FILE}")"
  fi
  info "Cluster fqdn: ${cluster_fqdn}"
  cluster_server_url="https://${cluster_fqdn}:6443"
  info "Cluster server url: ${cluster_server_url}"
  cluster_name=${cluster_fqdn%%.*}
  info "Cluster name: ${cluster_name}"

  # Setting cluster configuration in hadr kubeconfig
  kubectl config set-cluster "${cluster_name}" --server="${cluster_server_url}" --certificate-authority="${hadr_ca_crt}" \
    --kubeconfig="${ALTERNATE_CLUSTER_KUBECONFIG}" --embed-certs || error_with_code "15002"

  # Setting up user and associating with cert and key
  kubectl config set-credentials "${cert_user}" --client-certificate="${hadr_user_crt}" \
    --client-key="${cert_key}" --embed-certs --kubeconfig="${ALTERNATE_CLUSTER_KUBECONFIG}" || error_with_code "15003"

  # Setting up kube context
  kubectl config set-context "${hadr_context}" --cluster="${cluster_name}" --namespace=uipath --user="${cert_user}" \
    --kubeconfig="${ALTERNATE_CLUSTER_KUBECONFIG}" || error_with_code "15004"

  kubectl config use-context "${hadr_context}" --kubeconfig="${ALTERNATE_CLUSTER_KUBECONFIG}" || error_with_code "15005"
}

function retrieve_alternate_cluster_kubeconfig() {
  create_hadr_user
  echo
  echo "###################### ALTERNATE CLUSTER KUBECONFIG ######################"
  base64 < "${ALTERNATE_CLUSTER_KUBECONFIG}" | tr -d "\n"
  echo
  echo "##########################################################################"
  echo
  return 0
}

function create_hadr_user() {
  local k8s_csr_request_name="hadr-access"
  local key_length=2048
  local hadr_key
  local hadr_user hadr_context

  hadr_key="${HADR_CERTS_FOLDER_PATH}/hadr.key"
  hadr_user="hadr-user"

  rm -rf "${HADR_CERTS_FOLDER_PATH}" && mkdir -p "${HADR_CERTS_FOLDER_PATH}"
  info "Temp folder path: ${HADR_CERTS_FOLDER_PATH}"

  # Create an hadr key
  openssl genrsa -out "${hadr_key}" "${key_length}"

  # Create CSR
  create_user_csr "${k8s_csr_request_name}" "${HADR_CERTS_FOLDER_PATH}" "${hadr_key}" "${hadr_user}"

  # Approve CSR
  approve_csr "${k8s_csr_request_name}"

  create_custom_kubeconfig "${k8s_csr_request_name}" "${HADR_CERTS_FOLDER_PATH}" "${hadr_key}" "${hadr_user}"

  create_hadr_rolebinding "${HADR_CERTS_FOLDER_PATH}" "${hadr_user}"

  return 0
}