#!/bin/bash

if ! test -d /opt/UiPathAutomationSuite/UiPath_Installer ; then
  echo "Dir: /opt/UiPathAutomationSuite/UiPath_Installer does not exist"
fi
cd /opt/UiPathAutomationSuite/UiPath_Installer || exit 1

export KUBECONFIG=/etc/rancher/rke2/rke2.yaml PATH=$PATH:/var/lib/rancher/rke2/bin

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/utils.sh"

function create_certs_with_expiry() {
  local folder="$1"
  local dns_list="$2"
  local days="$3"
  local counter=1
  IFS=',' read -a dnsArray <<< "$dns_list"
  mkdir -p $1
  cd $1
  openssl genrsa -des3 -out rootCA.key -passout pass:12345 2048
  openssl req -x509 -new -nodes -key rootCA.key --passin pass:12345 -sha256 -days "${days}" -out rootCA.crt -subj "/C=US/ST=NY/O=UiPath, Inc./CN=UiPath, Service Fabric"
  
  echo -e "authorityKeyIdentifier=keyid,issuer\nbasicConstraints=CA:FALSE\nkeyUsage = digitalSignature, nonRepudiation, keyEncipherment, dataEncipherment\nextendedKeyUsage = serverAuth\nsubjectAltName = @alt_names\n[alt_names]" > v3.ext

  for i in "${dnsArray[@]}"
  do
    printf "%s\n" "DNS.$counter = $i" >> v3.ext
    counter=$((counter+1))
  done

  openssl req -new -nodes -out server.csr -newkey rsa:2048 -keyout server.key -subj "/C=US/ST=NY/O=UiPath, Inc./CN=Service Fabric"
  openssl x509 -req -in server.csr -CA rootCA.crt -CAkey rootCA.key  --passin pass:12345  -CAcreateserial -out server.crt -days "${days}" -sha256 -extfile v3.ext
  openssl pkcs12 -export -out identity.pfx -inkey rootCA.key -in rootCA.crt --passin pass:12345 -passout pass:12345
  cd ..
}    

function trigger_cert_updation_with_expiry() {
  local fqdn="$(jq -r ".fqdn" < /opt/UiPathAutomationSuite/UiPath_Installer/input.json)"
  create_certs_with_expiry "cert-expiration-tests" "${fqdn},*.${fqdn}" 2
  /opt/UiPathAutomationSuite/UiPath_Installer/configureUiPathAS.sh tls-cert update --ca-cert-file cert-expiration-tests/rootCA.crt --tls-cert-file cert-expiration-tests/server.crt --tls-key-file cert-expiration-tests/server.key -f
  delete_cert_exporter_pod
}

function delete_cert_exporter_pod() {
  mapfile -t cert_exporter_pod < <(kubectl get po -n cattle-monitoring-system | grep cert-exporter | awk '{print $1}')
  for pod in "${cert_exporter_pod[@]}"; do
    kubectl delete po "${pod}" -n cattle-monitoring-system || true
  done
}

function test_CertificateExpiration() {
  [[ "$(is_alert_fired "SecretCertificateExpiry7Days" "tls.crt" ".labels.key_name")" == "true" && "$(is_alert_fired "SecretCertificateExpiry7Days" "ca.crt" ".labels.key_name")" == "true" ]] && appendOutput "1660855" "true" || appendOutput "1660855" "false"
  [[ "$(is_alert_fired "SecretCertificateExpiry30Days" "tls.crt" ".labels.key_name")" == "true" && "$(is_alert_fired "SecretCertificateExpiry30Days" "ca.crt" ".labels.key_name")" == "true" ]] && appendOutput "1660854" "true" || appendOutput "1660854" "false"
}

