#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#GLOBALS:
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error "Could not determine modules path"
fi
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/upgrade_utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/argocd_utils.sh"

function run_ceph_compaction() {
  local rook_cluster_namespace=rook-ceph
  local index
  local try=20
  local is_compact_success
  local healthy_osds
  local osd_id
  healthy_osds=$(kubectl -n $rook_cluster_namespace exec deploy/rook-ceph-tools -- ceph osd df -f json  | jq '.nodes[] | select(.status=="up").id')
  for osd_id in $healthy_osds ; do
    is_compact_success="false"
    for index in $(seq 0 "${try}"); do
      if kubectl -n $rook_cluster_namespace exec deploy/rook-ceph-tools -- ceph tell "osd.${osd_id}" compact; then
        is_compact_success="true"
        break
      fi
      [[ $index -eq $try ]] || sleep 60
    done
    [[ $is_compact_success == "true" ]] || error "Failure while running compact on OSD.${osd_id}"
  done
}

function check_ceph_health() {
  local rook_cluster_namespace=rook-ceph
  current_rook_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"rook-version="}{.metadata.labels.rook-version}{"\n"}{end}' | sort | uniq)
  # current_ceph_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"ceph-version="}{.metadata.labels.ceph-version}{"\n"}{end}' | sort | uniq)
  if [[ "${current_rook_version}" == "rook-version=v1.5.9" ]];
  then
      # We are still on older version and we will be upgrading to rook v1.8.9-4.gb6f35a55f, hence validate ceph health first
      ceph_health_exit_on_error
      run_ceph_compaction
  fi
}

check_ceph_health
